package cc.winboll.studio.miniplayer;

/**
 * @Author ZhanGSKen<zhangsken@188.com>
 * @Date 2025/06/29 10:13
 * @Describe MidiPlayer
 */
import android.content.Context;
import android.media.midi.MidiDevice;
import android.media.midi.MidiDeviceInfo;
import android.media.midi.MidiInputPort;
import android.media.midi.MidiManager;
import android.os.Build;
import android.os.Handler;
import cc.winboll.studio.libappbase.LogUtils;
import java.io.File;
import java.io.IOException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class MidiPlayer {

    public static final String TAG = "MidiPlayer";

    private Context mContext;
    private MidiManager mMidiManager;
    private MidiDevice mMidiDevice;
    private MidiInputPort mInputPort;
    private ExecutorService mExecutor; // 线程池懒加载
    private Handler mHandler = new Handler();
    private MidiTrack[] mTracks; // 多轨道数组
    private boolean isPlaying = false;
    private int mCurrentTrack = -1; // 当前选中轨道（-1表示全部）
    private boolean isSynthConnected = false; // 合成器连接状态
    private OnSynthConnectedListener mConnectionListener; // 连接回调

    // 乐器库相关
    private SoundFontManager mSoundFontManager;

    // 连接回调接口
    public interface OnSynthConnectedListener {
        void onConnected(boolean success);
    }

    public MidiPlayer(Context context) {
        mContext = context;
        // 仅在Android 6.0+初始化MidiManager
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            mMidiManager = (MidiManager) context.getSystemService(Context.MIDI_SERVICE);
        }
        mSoundFontManager = new SoundFontManager(context);
    }

    // 设置连接回调
    public void setOnSynthConnectedListener(OnSynthConnectedListener listener) {
        mConnectionListener = listener;
    }

    // 懒加载线程池，确保每次使用时都是可用状态
    private ExecutorService getExecutor() {
        if (mExecutor == null || mExecutor.isShutdown() || mExecutor.isTerminated()) {
            mExecutor = Executors.newSingleThreadExecutor();
        }
        return mExecutor;
    }

    // 加载Midi文件
    public boolean loadMidiFile(File file) {
        try {
            MidiParser parser = new MidiParser(file);
            mTracks = parser.parse(); // 解析多轨道
            return mTracks != null && mTracks.length > 0;
        } catch (IOException e) {
            LogUtils.d(TAG, "加载Midi文件失败: " + e.getMessage());
            return false;
        }
    }

    // 加载自定义乐器（SoundFont）
    public boolean loadSoundFont(File soundFontFile) {
        return mSoundFontManager.loadSoundFont(soundFontFile);
    }

    // 启动播放
    public void start() {
        LogUtils.d(TAG, "start()");
        // 检查版本和状态
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
            LogUtils.d(TAG, "Midi播放需要Android 6.0及以上版本");
            notifyConnectionResult(false);
            return;
        }
        if (isPlaying || mTracks == null || mMidiManager == null) {
            return;
        }

        isPlaying = true;
        // 使用懒加载的线程池执行任务
        getExecutor().execute(new Runnable() {
				@Override
				public void run() {
					LogUtils.d(TAG, "getExecutor().execute(new Runnable()...");
					// 重置连接状态并尝试连接
					isSynthConnected = false;
					connectToSynth();

					// 等待连接完成（最多3秒超时）
					int waitCount = 0;
					while (!isSynthConnected && waitCount < 30) { // 30*100ms=3s
						try {
							Thread.sleep(100);
						} catch (InterruptedException e) {
							Thread.currentThread().interrupt();
							break;
						}
						waitCount++;
					}

					// 检查连接结果
					if (!isSynthConnected || mInputPort == null) {
						LogUtils.d(TAG, "合成器连接失败或输入端口为空");
						isPlaying = false;
						notifyConnectionResult(false);
						return;
					}

					// 连接成功，开始播放
					notifyConnectionResult(true);
					playTracks();
				}
			});
    }

    // 播放指定轨道（支持多轨同时播放）
    private void playTracks() {
        LogUtils.d(TAG, "playTracks()");
        LogUtils.d(TAG, String.format("mInputPort %s", mInputPort));
        if (mInputPort == null) {
            return;
        }
        LogUtils.d(TAG, String.format("isPlaying %s", isPlaying));

        while (isPlaying) {
            for (int i = 0; i < mTracks.length; i++) {
                // 控制是否播放当前轨道（支持单轨/多轨切换）
                if (mCurrentTrack == -1 || mCurrentTrack == i) {
                    MidiTrack track = mTracks[i];
                    if (track.hasNextEvent()) {
                        byte[] event = track.nextEvent();
                        try {
                            LogUtils.d(TAG, "mInputPort.send...");
                            mInputPort.send(event, 0, event.length);
                        } catch (IOException e) {
                            LogUtils.d(TAG, "发送Midi事件失败: " + e.getMessage(), Thread.currentThread().getStackTrace());
                        }
                    } else {
                        track.reset(); // 轨道播放完毕，重置
                    }
                }
            }
            // 控制播放速度（根据Midi事件的时间戳调整）
            try {
                Thread.sleep(10);
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
            }
        }
    }

    // 连接到系统Midi合成器（适配API 30，移除API 31+方法）
    private void connectToSynth() {
        if (mMidiManager == null) {
            return;
        }

        // 重置资源
        closeMidiResources();

        // 适配API 30及以下：直接遍历设备寻找合成器（移除getDefaultDeviceId调用）
        searchSynthInDevices();
    }

    // 遍历设备寻找合成器
    private void searchSynthInDevices() {
		MidiDeviceInfo[] devices = mMidiManager.getDevices();
		if (devices == null || devices.length == 0) {
			LogUtils.d(TAG, "未检测到任何MIDI设备");
			isSynthConnected = false;
			return;
		}

		// 遍历所有设备，优先找合成器类型，若无则尝试其他类型
		MidiDeviceInfo targetDevice = null;
		for (MidiDeviceInfo device : devices) {
			// 优先匹配合成器类型（3）
			if (device.getType() == 3) {
				targetDevice = device;
				break;
			}
			// 若未找到，尝试其他可能支持输入的设备（如通用MIDI设备）
			if (targetDevice == null && device.getInputPortCount() > 0) {
				targetDevice = device;
			}
		}

		if (targetDevice != null) {
			mMidiManager.openDevice(targetDevice, new MidiManager.OnDeviceOpenedListener() {
					@Override
					public void onDeviceOpened(MidiDevice device) {
						setupMidiDevice(device);
					}
				}, mHandler);
		} else {
			LogUtils.d(TAG, "未找到可用的MIDI设备（含合成器）");
			isSynthConnected = false;
		}
	}

    // 初始化MIDI设备并打开输入端口
    private void setupMidiDevice(MidiDevice device) {
        if (device == null) {
            LogUtils.d(TAG, "打开MIDI设备失败");
            isSynthConnected = false;
            return;
        }

        mMidiDevice = device;
        try {
            // 打开第一个输入端口（通常索引为0）
            mInputPort = mMidiDevice.openInputPort(0);
            if (mInputPort != null) {
                isSynthConnected = true;
                LogUtils.d(TAG, "成功打开MIDI输入端口");
            } else {
                LogUtils.d(TAG, "设备无可用输入端口");
                isSynthConnected = false;
            }
        } catch (Exception e) {
            LogUtils.d(TAG, "打开输入端口失败: " + e.getMessage());
            isSynthConnected = false;
        }
    }

    // 关闭MIDI资源
    private void closeMidiResources() {
        if (mInputPort != null) {
            try {
                mInputPort.close();
            } catch (IOException e) {
                LogUtils.d(TAG, "关闭输入端口失败: " + e.getMessage());
            }
            mInputPort = null;
        }
        if (mMidiDevice != null) {
            try {
                mMidiDevice.close();
            } catch (IOException e) {
                LogUtils.d(TAG, e, Thread.currentThread().getStackTrace());
            }
            mMidiDevice = null;
        }
    }

    // 通知连接结果
    private void notifyConnectionResult(final boolean success) {
        if (mConnectionListener != null) {
            mHandler.post(new Runnable() {
					@Override
					public void run() {
						mConnectionListener.onConnected(success);
					}
				});
        }
    }

    // 控制轨道（静音/独奏）
    public void setTrackMute(int trackIndex, boolean mute) {
        if (trackIndex >= 0 && trackIndex < mTracks.length) {
            mTracks[trackIndex].setMute(mute);
        }
    }

    // 切换当前播放轨道（-1表示全部）
    public void setCurrentTrack(int trackIndex) {
        mCurrentTrack = trackIndex;
    }

    // 获取轨道数量（供Adapter使用）
    public int getTrackCount() {
        return mTracks != null ? mTracks.length : 0;
    }

    // 暂停播放
    public void pause() {
        isPlaying = false;
    }

    // 停止播放并释放资源
    public void stop() {
        isPlaying = false;
        isSynthConnected = false; // 重置连接状态
        closeMidiResources(); // 释放MIDI资源

        // 终止线程池
        if (mExecutor != null) {
            mExecutor.shutdownNow();
            mExecutor = null;
        }
    }
}

